from logyca import buildUrl

def test_helpers():
    '''Description
    Checking the helpers
    '''
    url1='https://domain.com'
    url2='api/get'
    assert buildUrl(url1,url2)=='https://domain.com/api/get'    




from pydantic import BaseModel,Field

class APIKeyScheme(BaseModel):
    enable:bool=Field(default=True,description="Used to enable the check, if it is disabled the endpoint will log in without checking authentication")
    key:str=Field(default='',description="Header api key")
    


from urllib import parse

def buildUrl(url1:str,url2:str)->str:
    '''Description    
    Build a from url correctly
    :param url1: http(s)://initial/string
    :param url2: following url path
    :return str: url
    '''
    return parse.urljoin(url1, url2)

def convert_string_to_boolean(truth_value: str) -> bool:
    lowercased_value = str(truth_value).lower()

    if lowercased_value in ("yes", "true", "1", "on"):
        return True
    elif lowercased_value in ("no", "false", "0", "off"):
        return False
    else:
        raise ValueError(f"Invalid boolean string value: {truth_value}"
        )


from pydantic import BaseModel,Field

class OAuthTokenScheme(BaseModel):
    enable:bool=Field(default=True,description="Used to enable the check, if it is disabled the endpoint will log in without checking authentication")
    endpoint_azure_b2c_apicrud:str=Field(default=True,description="enpoint azure b2c apicrudurl")
    endpoint_azure_b2c_apicrud_method_validate_login:str=Field(default=True,description="endpoint azure b2c apicrud method for  validate login")
    logyca_apikey:str=Field(default='',description="Api key for platform")
    


from logyca import convertDateTimeStampUTCtoUTCColombia

def test_convertDateTimeStampUTCtoUTCColombia():
    ts = 1571595618.0
    assert str(convertDateTimeStampUTCtoUTCColombia(ts))=='2019-10-20 13:20:18-05:00'  



from logyca import HealthEnum, LogycaStatusEnum, APIResultDTO, ApiFilterExceptionDTO, HTTPExceptionDTO, HealthDTO, TokensDTO

def test_health_dto():
    '''Description
    Checking the data types and the defined content of the enums
    '''
    listHealth=[]

    listHealth.append(HealthDTO(name='Check CPU',status=HealthEnum.Ok,description='OK'))
    listHealth.append(HealthDTO(name='Check Connect DB',status=HealthEnum.Warning,description='Warning'))
    listHealth.append(HealthDTO(name='Check Connect Storage',status=HealthEnum.Critical,description='Critical'))

    for item in listHealth:
        if (item.description=='OK'):
            assert item.status==0
        if (item.description=='Warning'):
            assert item.status==1
        if (item.description=='Critical'):
            assert item.status==2

def test_api_result():
    '''Description
    Checking the data types and schema structure
    '''
    tokensDTO=TokensDTO()
    tokensDTO.token='Token Example'

    apiFilterExceptionDTO=ApiFilterExceptionDTO()
    apiFilterExceptionDTO.isError=False
    apiFilterExceptionDTO.logycaStatus=LogycaStatusEnum.Already_Exists
    apiFilterExceptionDTO.status=LogycaStatusEnum.Already_Exists.mappingHttpStatusCode

    assert apiFilterExceptionDTO.logycaStatus == 6
    assert apiFilterExceptionDTO.status == 409

    httpExceptionDTO=HTTPExceptionDTO()
    httpExceptionDTO.detail='No Problem'

    listHealth=[]

    listHealth.append(HealthDTO(name='Check CPU',status=HealthEnum.Ok,description='OK'))
    listHealth.append(HealthDTO(name='Check Connect DB',status=HealthEnum.Warning,description='Warning'))
    listHealth.append(HealthDTO(name='Check Connect Storage',status=HealthEnum.Critical,description='Critical'))


    apiResultDTO=APIResultDTO()
    apiResultDTO.resultMessage=httpExceptionDTO.detail
    apiResultDTO.resultObject=listHealth
    apiResultDTO.dataError=False
    apiResultDTO.resultToken=tokensDTO
    apiResultDTO.apiException=apiFilterExceptionDTO

    for item in apiResultDTO.resultObject:
        if (item.description=='OK'):
            assert item.status==0
        if (item.description=='Warning'):
            assert item.status==1
        if (item.description=='Critical'):
            assert item.status==2




from fastapi import Depends,HTTPException
from fastapi.security import HTTPBearer
from fastapi.encoders import jsonable_encoder
from logyca.utils.helpers.stringshelpers import buildUrl
from logyca.utils.helpers.datetimehelpers import convertDateTimeStampUTCtoUTCColombia
from logyca.utils.constants.logycastatusenum import LogycaStatusEnum
from logyca.schemas.output.apIresultdto import APIResultDTO
from logyca.schemas.input.oauth_token import OAuthTokenScheme
import jwt
import aiohttp
from logyca.schemas.input.claimsdto import ClaimsDTO
from logyca.schemas.input.jwt import BearerToken

auth_scheme = HTTPBearer()

class OAuthToken:
    '''Description

    ## Example of use
    from fastapi import FastAPI, Depends
    from logyca import OAuthTokenScheme, OAuthToken, ClaimsDTO
    import os

    ENPOINTAZUREB2C_APICRUDURL=os.getenv('APICRUD_URL','https://domain.ccc/')
    ENPOINTAZUREB2C_APICRUDMETHODVALIDATELOGIN=os.getenv('METHOD_VALIDATE_LOGIN','api/method/')
    LOGYCA_APIKEY=os.getenv('LOGYCA_APIKEY','...key...')

    app = FastAPI()

    settings_oauth_token=OAuthTokenScheme(
        endpoint_azure_b2c_apicrud=ENPOINTAZUREB2C_APICRUDURL,
        endpoint_azure_b2c_apicrud_method_validate_login=ENPOINTAZUREB2C_APICRUDMETHODVALIDATELOGIN,
        logyca_apikey=LOGYCA_APIKEY,
        enable=True)
    get_oauth_token = OAuthToken(settings_oauth_token)

    @app.get("/data/")
    def read_item(claims_dto: ClaimsDTO = Depends(get_oauth_token)):
        return {"user checked ok: ": f"{claims_dto}"}
    '''
    
    def __init__(self,oauth_token_scheme:OAuthTokenScheme):
        self.enable=oauth_token_scheme.enable
        self.endpoint_azure_b2c_apicrud=oauth_token_scheme.endpoint_azure_b2c_apicrud
        self.endpoint_azure_b2c_apicrud_method_validate_login=oauth_token_scheme.endpoint_azure_b2c_apicrud_method_validate_login
        self.logyca_apikey=oauth_token_scheme.logyca_apikey
    
    async def __call__(self,bearer_token:BearerToken = Depends(auth_scheme)):
        '''Description
        Used by fastapi dependency injection
        '''
        try:
            claimsDTO=ClaimsDTO()
            if self.enable is False:
                return claimsDTO
            else:
                headers={
                    'Authorization': 'Bearer {0}'.format(bearer_token.credentials),
                    'ApiKey': self.logyca_apikey
                    }
                urlEndpoint=buildUrl(self.endpoint_azure_b2c_apicrud,self.endpoint_azure_b2c_apicrud_method_validate_login)
                async with aiohttp.ClientSession(headers=headers) as session:
                    async with session.get(urlEndpoint) as response:
                        json_body = await response.json()
                payload = jwt.decode(
                    bearer_token.credentials,
                    algorithms=["RS256"],
                    options={"verify_signature":False}
                    )            
                claimsDTO.name=payload["name"]
                claimsDTO.email=payload["emails"][0]
                claimsDTO.expirationTimeUTCBogota=convertDateTimeStampUTCtoUTCColombia(payload["exp"])
                return claimsDTO
        except Exception as e:
            msgErrorFromSession=response.headers.get('www-authenticate')
            aPIResultDTO=APIResultDTO()
            aPIResultDTO.resultMessage=msgErrorFromSession
            aPIResultDTO.apiException.logycaStatus=LogycaStatusEnum.UnAuthenticated
            aPIResultDTO.apiException.status=LogycaStatusEnum.UnAuthenticated.mappingHttpStatusCode
            raise HTTPException(
                detail=jsonable_encoder(aPIResultDTO),
                status_code=aPIResultDTO.apiException.status
                )





from logyca.utils.constants.logycastatusenum import LogycaStatusEnum
from logyca.utils.constants.app import App

from logyca.schemas.input.claimsdto import ClaimsDTO
from logyca.schemas.input.api_key import APIKeyScheme
from logyca.schemas.input.jwt import BearerToken
from logyca.schemas.input.oauth_token import OAuthTokenScheme 

from logyca.schemas.output.healthdto import HealthEnum
from logyca.schemas.output.apifilterexceptiondto import ApiFilterExceptionDTO
from logyca.schemas.output.apIresultdto import APIResultDTO
from logyca.schemas.output.healthdto import HealthDTO
from logyca.schemas.output.httpexceptiondto import HTTPExceptionDTO
from logyca.schemas.output.tokensdto import TokensDTO

from logyca.utils.helpers.datetimehelpers import convertDateTimeStampUTCtoUTCColombia
from logyca.utils.helpers.stringshelpers import buildUrl
from logyca.utils.helpers.stringshelpers import convert_string_to_boolean

from logyca.utils.handlers.logger import Logger
from logyca.utils.handlers.logger import ConstantsLogger

# If you need the logyca library but without the utilities focused on fastapi
try:
    from logyca.dependencies.api_key_simple_auth import APIKey
    from logyca.dependencies.oauth_token import OAuthToken
    from logyca.utils.handlers.exception_handlers import validation_exception_handler_async
    from logyca.utils.handlers.exception_handlers import http_exception_handler_async
    from logyca.utils.handlers.exception_handlers import unhandled_exception_handler_async
except ImportError:
    APIKey = None
    OAuthToken = None
    validation_exception_handler_async = None
    http_exception_handler_async = None
    unhandled_exception_handler_async = None



from pydantic import BaseModel,Field
from logyca.utils.constants.logycastatusenum import LogycaStatusEnum
from typing import Any
from http import HTTPStatus

class ApiFilterExceptionDTO(BaseModel):
        message:str=Field(default='',description="Gets or sets error message")
        isError:bool=Field(default=True,description="Gets or sets a value indicating whether api has error")
        detail:Any=Field(default=None,description="Gets or sets error detail")
        status:HTTPStatus=Field(default=HTTPStatus.OK,description="Gets or sets error code")
        logycaStatus:LogycaStatusEnum=Field(default=LogycaStatusEnum.Ok,description="Gets or sets error code")
        def __init__(self, **kwargs):
                kwargs['isError'] = False
                kwargs['message'] = ''
                kwargs['status'] = HTTPStatus.OK
                super().__init__(**kwargs)



from pydantic import BaseModel,Field

class ClaimsDTO(BaseModel):
    name:str=Field(default='',description="User name")
    email:str=Field(default='',description="Email from user")
    expirationTimeUTCBogota:str=Field(default='',description="Expiration data from user token")



import os, logging
from enum import StrEnum

StrPath = str | os.PathLike[str]

class ConstantsLogger(StrEnum):
    NAME = "LoggerContext"

class Logger:
    """
    # Example of using Logger
    At the root of the project, the logs folder is created and the types of errors are differentiated by different files.
    ```python

    # main.py
    from logyca import Logger, ConstantsLogger
    logger = Logger(logger_name=ConstantsLogger.NAME,log_dir=FOLDER_LOGS,log_file_name=f"{App.Settings.NAME}")
    logger.info(f"message")

    # Other files.py
    from logyca import Logger, ConstantsLogger
    import logging
    logger = logging.getLogger(ConstantsLogger.NAME)

    logger.info(f"message")
    logger.error(f"message")

    ```
    """
    def __init__(
        self,
        logger_name: str,
        log_dir: str,
        log_file_name: str
    ):
        os.makedirs(log_dir, exist_ok=True)
        self.log_dir = log_dir
        self.log_file_name = log_file_name
        self.logger_name = logger_name
        self.logger = logging.getLogger(logger_name)
        self.logger.setLevel(logging.DEBUG)
        self._setup_handlers()

    def _setup_handlers(self) -> None:
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        log_levels = (
            logging.DEBUG,
            logging.INFO,
            logging.WARNING,
            logging.ERROR,
            logging.CRITICAL,
        )
        log_names = ("debug", "info", "warning", "error", "critical")

        for level, name in zip(log_levels, log_names):
            log_file = os.path.join(self.log_dir, f"{self.log_file_name}.{name}.log")
            handler = logging.FileHandler(log_file)
            handler.setFormatter(formatter)
            handler.setLevel(level)
            handler.filter = lambda record, level=level: record.levelno == level
            self.logger.addHandler(handler)

    def debug(self, message: object) -> None:
        self.logger.debug(message)

    def info(self, message: object) -> None:
        self.logger.info(message)

    def warning(self, message: object) -> None:
        self.logger.warning(message)

    def error(self, message: object) -> None:
        self.logger.error(message)

    def critical(self, message: object) -> None:
        self.logger.critical(message)



from logyca.schemas.input.api_key import APIKeyScheme
from fastapi import Security, HTTPException
from fastapi.security.api_key import APIKeyHeader
from starlette.status import HTTP_403_FORBIDDEN

header_value="x-api-key"

auth_scheme_api_key_header = APIKeyHeader(name=header_value, auto_error=False)

class APIKey:
    '''Description

    ## Example of use
    from fastapi import FastAPI, Depends
    from logyca import APIKeyScheme, APIKey
    import os

    APY_KEY=os.getenv('APY_KEY','password_key')

    app = FastAPI()

    settings_api_key=APIKeyScheme(key=APY_KEY, enable=True)
    get_api_key = APIKey(settings_api_key)

    @app.get("/data/")
    def read_item(api_key: str = Depends(get_api_key)):
        return {"api_key checked ok: ": f"{api_key}"}
    '''    
    def __init__(self,api_key_scheme:APIKeyScheme):
        self.api_key=api_key_scheme.key
        self.check_enable=api_key_scheme.enable
    
    async def __call__(self,api_key_header: str = Security(auth_scheme_api_key_header)):
        '''Description
        Used by fastapi dependency injection
        '''
        if self.check_enable is False:
            return ''
        elif api_key_header == self.api_key:
            return api_key_header   
        else:
            raise HTTPException(
                status_code=HTTP_403_FORBIDDEN, detail="Could not validate API KEY"
            )



from logyca.utils.constants.healthenum import HealthEnum

class HealthDTO():
    '''Description
    LOGYCA HTTP Custom States
    :param name:str Gets or sets health status name
    :param status:HealthEnum Gets or sets health check status response code
    :param description:str Gets or sets description of the monitoring process
    :return int: Integer according to variable meaning
    '''
    name:str
    status:HealthEnum
    description:str
    def __init__(self, name='', status=HealthEnum.Ok,description=''):
        self.name = name
        self.status = status
        self.description = description


from logyca.schemas.output.tokensdto import TokensDTO
from logyca.schemas.output.apifilterexceptiondto import ApiFilterExceptionDTO
from typing import Any
from pydantic import BaseModel,Field

class APIResultDTO(BaseModel):
        resultToken:TokensDTO=Field(default=TokensDTO(),description="Gets or sets object with result")
        resultObject:Any=Field(default=None,description="Gets or sets object with result")
        apiException:ApiFilterExceptionDTO=Field(description="Gets or sets error")
        resultMessage:str=Field(default='',description="Gets or sets result of negative or positive message")
        dataError:bool=Field(default=True,description="Gets or sets a value indicating whether gets or sets a value if it is data error")
        def __init__(self, **kwargs):
                kwargs['dataError'] = False
                kwargs['apiException'] = ApiFilterExceptionDTO()
                super().__init__(**kwargs)



from datetime import datetime, timezone
import pytz
from logyca.utils.constants.app import App

def convertDateTimeStampUTCtoUTCColombia(timestamp)->datetime:
    '''Description    
    Build a from url correctly
    :param timestamp:int: timestamp as timezone UTC
    :return datetime: datetime as timezone UTC(-5) Colombia
    '''
    timestamp_utc=datetime.fromtimestamp(timestamp, tz=timezone.utc)
    dateTimeColombia = timestamp_utc.astimezone(pytz.timezone(App.TimeZone.Colombia))
    return dateTimeColombia




#!/usr/bin/env python
from setuptools import setup, find_packages

COMPANY_NAME="LOGYCA"
PACKAGE_NAME = "logyca"
VERSION = "0.1.10rc1"

install_requires = ["pydantic >=1.8","pytz >=2023.3"]
install_requires_fastapi = ["fastapi>=0.96.0","starlette>=0.24.0"]
install_requires_oauth_token = install_requires_fastapi + ["aiohttp>=3.8.5","jwt>=1.1.0"]
install_requires_api_key_simple_auth = install_requires_fastapi

extras_require = {
    "oauth_token": install_requires + install_requires_oauth_token,
    "api_key_simple_auth": install_requires + install_requires_api_key_simple_auth,
    "oauth_token-api_key_simple_auth": install_requires + install_requires_oauth_token + install_requires_api_key_simple_auth,
}

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=f'This package name is reserved by {COMPANY_NAME} company',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    license='MIT License',
    author='Jaime Andres Cardona Carrillo',
    author_email='jacardona@outlook.com',
    url='https://github.com/logyca/<soon>',
    keywords="api result, result dto, result scheme",
    classifiers=[
    "Development Status :: 4 - Beta",
    "Environment :: Web Environment",
    "Framework :: Pydantic :: 1",
    "Framework :: Pydantic",
    "Intended Audience :: Developers",
    "Intended Audience :: Information Technology",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python",
    "Topic :: Internet :: WWW/HTTP :: HTTP Servers",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Internet",
    "Topic :: Software Development :: Libraries :: Application Frameworks",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development",
    "Typing :: Typed",
    ],
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=install_requires,
    extras_require=extras_require,
)



from logyca.utils.handlers.logger import ConstantsLogger
from fastapi.exceptions import RequestValidationError, HTTPException
from fastapi.requests import Request
from fastapi.responses import JSONResponse
from logyca.utils.constants.logycastatusenum import LogycaStatusEnum
from logyca.schemas.output.apIresultdto import APIResultDTO
import json, traceback
import logging
from starlette.status import (
    HTTP_200_OK,
    HTTP_201_CREATED,
    HTTP_202_ACCEPTED,
    HTTP_400_BAD_REQUEST,
    HTTP_401_UNAUTHORIZED,
    HTTP_403_FORBIDDEN,
    HTTP_404_NOT_FOUND,
    HTTP_409_CONFLICT,
    HTTP_429_TOO_MANY_REQUESTS,
    HTTP_500_INTERNAL_SERVER_ERROR,
    HTTP_501_NOT_IMPLEMENTED,
    HTTP_503_SERVICE_UNAVAILABLE,
    HTTP_504_GATEWAY_TIMEOUT,
)

"""
This event handler is tested in fastapi to catch all exceptions and give the result in APIResult

Example:

from fastapi import FastAPI
from fastapi.exceptions import HTTPException, RequestValidationError
from logyca.utils.handlers.exception_handlers import http_exception_handler_async, unhandled_exception_handler_async,validation_exception_handler_async

app = FastAPI()

app.add_exception_handler(RequestValidationError, validation_exception_handler_async)
app.add_exception_handler(HTTPException, http_exception_handler_async)
app.add_exception_handler(Exception, unhandled_exception_handler_async)

"""


logger = logging.getLogger(ConstantsLogger.NAME)

def mapping_logyca_status_code(status_code: int):
    status_mapping = {
        HTTP_200_OK: LogycaStatusEnum.Ok,
        HTTP_201_CREATED: LogycaStatusEnum.Created,
        HTTP_202_ACCEPTED: LogycaStatusEnum.Partial,
        HTTP_400_BAD_REQUEST: LogycaStatusEnum.Invalid_Argument,
        HTTP_401_UNAUTHORIZED: LogycaStatusEnum.UnAuthenticated,
        HTTP_403_FORBIDDEN: LogycaStatusEnum.Permission_Denied,
        HTTP_404_NOT_FOUND: LogycaStatusEnum.Not_Found,
        HTTP_409_CONFLICT: LogycaStatusEnum.Already_Exists,
        HTTP_429_TOO_MANY_REQUESTS: LogycaStatusEnum.Resource_Exhausted,
        HTTP_500_INTERNAL_SERVER_ERROR: LogycaStatusEnum.Internal,
        HTTP_501_NOT_IMPLEMENTED: LogycaStatusEnum.UnImplemented,
        HTTP_503_SERVICE_UNAVAILABLE: LogycaStatusEnum.UnAvailable,
        HTTP_504_GATEWAY_TIMEOUT: LogycaStatusEnum.DeadLine_Exceeded,
    }
    return status_mapping.get(status_code, LogycaStatusEnum.Not_Found)


def api_result_builder(
    detail: str, status_code: int, logyca_status: LogycaStatusEnum
) -> APIResultDTO:
    api_result_dto = APIResultDTO()
    api_result_dto.dataError = True
    api_result_dto.apiException.isError = True
    api_result_dto.apiException.status = status_code
    api_result_dto.apiException.logycaStatus = logyca_status
    api_result_dto.apiException.message = detail
    api_result_dto.apiException.detail = detail
    api_result_dto.resultMessage = detail
    return api_result_dto

async def validation_exception_handler_async(_: Request, exc: RequestValidationError):
    errors = []
    for error in exc.errors():
        if "type" in error and error["type"] == "json_invalid":
            errors.append({"field": "body", "msg": error["msg"]})
        elif len(error["loc"]) > 1:
            errors.append({"field": error["loc"][1], "msg": error["msg"]})
        elif len(error["loc"]) > 0:
            errors.append({"field": error["loc"][0], "msg": error["msg"]})
    api_result_dto = api_result_builder(
        detail=f"{errors}",
        status_code=HTTP_400_BAD_REQUEST,
        logyca_status=LogycaStatusEnum.Invalid_Argument,
    )
    return JSONResponse(
        status_code=200,
        content=api_result_dto.model_dump(),
    )

async def http_exception_handler_async(_: Request, exc: HTTPException):
    api_result_dto = api_result_builder(
        detail=exc.detail,
        status_code=exc.status_code,
        logyca_status=mapping_logyca_status_code(exc.status_code),
    )
    return JSONResponse(
        status_code=200,
        content=api_result_dto.model_dump(),
    )

async def unhandled_exception_handler_async(_: Request, exc: Exception):
    exception_info = {
        "error_type": type(exc).__name__,
        "message": str(exc),
        "traceback": traceback.format_exc(),
    }
    json_data: str = json.dumps(exception_info)
    logger.critical(json_data)
    api_result_dto = api_result_builder(
        detail=str(exc),
        status_code=HTTP_500_INTERNAL_SERVER_ERROR,
        logyca_status=LogycaStatusEnum.Unknown,
    )
    return JSONResponse(
        status_code=200,
        content=api_result_dto.model_dump(),
    )



from enum import IntEnum

class HealthEnum(IntEnum):
        '''Description
        Health check status result
        :param OK: The method was able to check the service and it appeared to be functioning properly
        :param Warning: The method was able to check the service but it appeared to be above some "warning" threshold or did not appear to be working properly
        :param Critical: The method detected that either the service was not running or it was above some "critical" threshold
        :param Unknown: The method cannot answer a valid test
        :return int: Integer according to variable meaning
        '''
        Ok = 0
        Warning = 1
        Critical = 2
        Unknown = 3



from enum import IntEnum
from http import HTTPStatus

class LogycaStatusEnum(IntEnum):
        '''Description
        LOGYCA Custom States
        \n:param Ok: Not an error; returned on success
        \n:param Cancelled: The operation was cancelled, typically by the caller.
        \n:param Unknown: Unknown error. For example, this error may be returned when a Status value received from another address space belongs to an error space that is not known in this address space. Also errors raised by APIs that do not return enough error information may be converted to this error.
        \n:param Invalid_Argument: The client specified an invalid argument. Note that this differs from FAILED_PRECONDITION. INVALID_ARGUMENT indicates arguments that are problematic regardless of the state of the system (e.g., a malformed file name).
        \n:param DeadLine_Exceeded: The deadline expired before the operation could complete. For operations that change the state of the system, this error may be returned even if the operation has completed successfully. For example, a successful response from a server could have been delayed long
        \n:param Not_Found: Some requested entity (e.g., file or directory) was not found. Note to server developers: if a request is denied for an entire class of users, such as gradual feature rollout or undocumented whitelist, NOT_FOUND may be used. If a request is denied for some users within a class of users, such as user-based access control, PERMISSION_DENIED must be used.
        \n:param Already_Exists: The entity that a client attempted to create (e.g., file or directory) already exists.
        \n:param Permission_Denied: The caller does not have permission to execute the specified operation. PERMISSION_DENIED must not be used for rejections caused by exhausting some resource (use RESOURCE_EXHAUSTEDinstead for those errors). PERMISSION_DENIED must not be used if the caller can not be identified (use UNAUTHENTICATED instead for those errors). This error code does not imply the request is valid or the requested entity exists or satisfies other pre-conditions.
        \n:param Resource_Exhausted: Some resource has been exhausted, perhaps a per-user quota, or perhaps the entire file system is out of space.
        \n:param Failed_Condition: The operation was rejected because the system is not in a state required for the operation's execution. For example, the directory to be deleted is non-empty, an rmdir operation is applied to a non-directory, etc. Service implementors can use the following guidelines to decide between FAILED_PRECONDITION, ABORTED, and UNAVAILABLE: (a) Use UNAVAILABLE if the client can retry just the failing call. (b) Use ABORTED if the client should retry at a higher level (e.g., when a client-specified test-and-set fails, indicating the client should restart a read-modify-write sequence). (c) Use FAILED_PRECONDITION if the client should not retry until the system state has been explicitly fixed. E.g., if an "rmdir" fails because the directory is non-empty, FAILED_PRECONDITIONshould be returned since the client should not retry unless the files are deleted from the directory.
        \n:param Aborted: The operation was aborted, typically due to a concurrency issue such as a sequencer check failure or transaction abort. See the guidelines above for deciding between FAILED_PRECONDITION, ABORTED, and UNAVAILABLE.
        \n:param Out_Of_Range: The operation was attempted past the valid range. E.g., seeking or reading past end-of-file. Unlike INVALID_ARGUMENT, this error indicates a problem that may be fixed if the system state changes. For example, a 32-bit file system will generate INVALID_ARGUMENT if asked to read at an offset that is not in the range [0,2^32-1], but it will generate OUT_OF_RANGE if asked to read from an offset past the current file size. There is a fair bit of overlap between FAILED_PRECONDITION and OUT_OF_RANGE. We recommend using OUT_OF_RANGE (the more specific error) when it applies so that callers who are iterating through a space can easily look for an OUT_OF_RANGE error to detect when they are done.
        \n:param UnImplemented: The operation is not implemented or is not supported/enabled in this service.
        \n:param Internal: Internal errors. This means that some invariants expected by the underlying system have been broken. This error code is reserved for serious errors.
        \n:param UnAvailable: The service is currently unavailable. This is most likely a transient condition, which can be corrected by retrying with a backoff.
        \n:param DataLoss: Unrecoverable data loss or corruption.
        \n:param UnAuthenticated: The request does not have valid authentication credentials for the operation.
        \n:param Partial: Saved partial
        \n:param Created: Pending
        \n:param InProcess: Transaction in process.
        \n:return int: Integer according to variable meaning
        '''
        Ok = 0
        Cancelled = 1
        Unknown = 2
        Invalid_Argument = 3
        DeadLine_Exceeded = 4
        Not_Found = 5
        Already_Exists = 6
        Permission_Denied = 7
        Resource_Exhausted = 8
        Failed_Condition = 9
        Aborted = 10
        Out_Of_Range = 11
        UnImplemented = 12
        Internal = 13
        UnAvailable = 14
        DataLoss = 15
        UnAuthenticated = 16
        Partial = 20
        Created = 1001
        InProcess = 1002
        @property
        def mappingHttpStatusCode(self):
                if (self.value==LogycaStatusEnum.Ok):                   return HTTPStatus.OK
                if (self.value==LogycaStatusEnum.Cancelled):            return HTTPStatus.NOT_FOUND
                if (self.value==LogycaStatusEnum.Unknown):              return HTTPStatus.INTERNAL_SERVER_ERROR
                if (self.value==LogycaStatusEnum.Invalid_Argument):     return HTTPStatus.BAD_REQUEST
                if (self.value==LogycaStatusEnum.DeadLine_Exceeded):    return HTTPStatus.GATEWAY_TIMEOUT
                if (self.value==LogycaStatusEnum.Not_Found):            return HTTPStatus.NOT_FOUND
                if (self.value==LogycaStatusEnum.Already_Exists):       return HTTPStatus.CONFLICT
                if (self.value==LogycaStatusEnum.Permission_Denied):    return HTTPStatus.FORBIDDEN
                if (self.value==LogycaStatusEnum.Resource_Exhausted):   return HTTPStatus.TOO_MANY_REQUESTS
                if (self.value==LogycaStatusEnum.Failed_Condition):     return HTTPStatus.BAD_REQUEST
                if (self.value==LogycaStatusEnum.Aborted):              return HTTPStatus.CONFLICT
                if (self.value==LogycaStatusEnum.Out_Of_Range):         return HTTPStatus.BAD_REQUEST
                if (self.value==LogycaStatusEnum.UnImplemented):        return HTTPStatus.NOT_IMPLEMENTED
                if (self.value==LogycaStatusEnum.Internal):             return HTTPStatus.INTERNAL_SERVER_ERROR
                if (self.value==LogycaStatusEnum.UnAvailable):          return HTTPStatus.SERVICE_UNAVAILABLE
                if (self.value==LogycaStatusEnum.DataLoss):             return HTTPStatus.INTERNAL_SERVER_ERROR
                if (self.value==LogycaStatusEnum.UnAuthenticated):      return HTTPStatus.UNAUTHORIZED
                if (self.value==LogycaStatusEnum.Partial):              return HTTPStatus.ACCEPTED
                if (self.value==LogycaStatusEnum.Created):              return HTTPStatus.CREATED
                if (self.value==LogycaStatusEnum.InProcess):            return HTTPStatus.ACCEPTED
                return HTTPStatus.NOT_FOUND



from pydantic import BaseModel,Field

class TokensDTO(BaseModel):
    token:str=Field(default='',description="Gets or sets result Token")
    refreshToken:str=Field(default='',description="Gets or sets Refresh token")
    result:str=Field(default='',description="Gets or sets result request")
    emailActiveDirectory:str=Field(default='',description="Gets or sets email Active Directory")
    message:str=Field(default='',description="Gets or sets user message")
